#define MAIN
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string.h>
#include "structs.h"
#include "functions.h"
#include <time.h>
void updateMap(Cell map [GSIZE][GSIZE], Interface *inter) {
	printf("New turn!\n");
	for (int i = 0; i < inter->width; ++i) 
		for (int j = 0; j < inter->height; ++j) {
			if (map[i][j].planet)
				planet(&map[i][j]);
			if (map[i][j].ship)
				ship(&map[i][j]);
			map[i][j].selected = 0;
		}

}

void intr2cell(Cell *cell, Interface *inter) {
	if (!cell) return;
	if (cell->planet) {
		inter->bars[BPRODUCTION] = inter->cursor->planet->production/10.0;
		inter->bars[BPROPULSION] = inter->cursor->planet->speed * 10;
		inter->bars[BWEAPONS] = inter->cursor->planet->attack * 10;
		inter->bars[BDEFENSES] = inter->cursor->planet->defend * 10;
		inter->bars[BSCIENCE] = inter->cursor->planet->science / 10;
	} else if (cell->ship) {
		inter->bars[BPRODUCTION] = 0;
		inter->bars[BPROPULSION] = inter->cursor->ship->speed * 10;
		inter->bars[BWEAPONS] = inter->cursor->ship->attack * 10;
		inter->bars[BDEFENSES] = inter->cursor->ship->defend * 10;
		inter->bars[BSCIENCE] = 0;
	} else {
		for (int i = 0; i < 5; ++i)
			inter->bars[i] = 0;
	}
}

void handleInterfaceClick(Interface *inter, int y) {
	if (!inter->cursor) return;
	if (y < 384 || y > 608)
		return;
	float pp;
	inter->selected = (y-384)/32 + 1;
	switch (inter->selected) {
		case OSCRAP:
			if (!inter->cursor->planet) {
				printf("Ships can only be decomissioned on planets.\n");
				return;
			}
			if (!inter->cursor->ship) {
				printf("There isn't a ship orbiting %s.\n", inter->cursor->planet->name);
				return;
			}
			if (inter->cursor->ship->race != inter->player) {
				printf("You cannot tell enemy ships what to do.\n");
				return;
			}
			if (inter->cursor->ship->race != inter->cursor->planet->race) {
				printf("Although the citizens of %s would like to scrap %s,"
				       "\nwhich is blockading their planet, they can't.\n", 
				       inter->cursor->planet->name,
				       inter->cursor->ship->name
				);
				return;
			}
			inter->cursor->planet->production += (inter->cursor->ship->transport? 100 : 75);
			printf("The Ex-%s was broken up for scrap on %s.\n",
					inter->cursor->ship->name,
					inter->cursor->planet->name
			      );
			free(inter->cursor->ship);
			inter->bars[BPRODUCTION] = inter->cursor->planet->production/10.0;
			inter->cursor->ship = NULL;
		break;
		case OUPGRADE:
			if (!inter->cursor->planet) {
				printf("Ships can only be upgraded on planets.\n");
				return;
			}
			if (!inter->cursor->ship) {
				printf("There isn't a ship orbiting %s.\n", inter->cursor->planet->name);
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("The %s planet %s is not stupid enough to upgrade\n"
				       "your ship, which is beseiging it.\n", 
				       inter->cursor->planet->race->adjective,
				       inter->cursor->planet->name
				);
				return;
			}
			if (inter->cursor->ship->race != inter->player) {
				printf("Your citizens on %s aren't stupid enough to upgrade\n"
				       "the enemy ship %s, which is blockading their planet.\n", 
				       inter->cursor->planet->name,
				       inter->cursor->ship->name
				);
				return;
			}
			if (inter->cursor->ship->attack >= inter->cursor->planet->attack) {
				printf("The shipyards of %s can't make any improvement on %s's weapons.\n",
						inter->cursor->planet->name,
						inter->cursor->ship->name
				      );
			} else {
				pp = (inter->cursor->planet->attack - inter->cursor->ship->attack)*10;
				if (pp > inter->cursor->planet->production) 
					printf("Insuffecent production points to upgrade weapons.\n");
				else {
					inter->cursor->planet->production -= pp;
					inter->cursor->ship->attack = inter->cursor->planet->attack;
					printf("%s's weapons upgraded to level %d.\n",
							inter->cursor->ship->name,
							inter->cursor->ship->attack
							);

				}
			}
			if (inter->cursor->ship->defend >= inter->cursor->planet->defend) {
				printf("The shipyards of %s can't make any improvement on %s's defenses.\n",
						inter->cursor->planet->name,
						inter->cursor->ship->name
				      );
			} else {
				pp = (inter->cursor->planet->defend - inter->cursor->ship->defend)*10;
				if (pp > inter->cursor->planet->production) 
					printf("Insuffecent production points to upgrade defenses.\n");
				else {
					inter->cursor->planet->production -= pp;
					inter->cursor->ship->defend = inter->cursor->planet->defend;
					printf("%s's defenses upgraded to level %d. %d HP.\n", 
							inter->cursor->ship->name,
							inter->cursor->ship->defend,
							inter->cursor->ship->mHP
							);

				}
				inter->cursor->ship->mHP += pp;
			}
			if (inter->cursor->ship->speed >= inter->cursor->planet->speed) {
				printf("The shipyards of %s can't make any improvement on %s's engines.\n",
						inter->cursor->planet->name,
						inter->cursor->ship->name
				      );
			} else {
				pp = (inter->cursor->planet->speed - inter->cursor->ship->speed)*10;
				if (pp > inter->cursor->planet->production) 
					printf("Insuffecent production points to upgrade engines.\n");
				else {
					inter->cursor->planet->production -= pp;
					inter->cursor->ship->speed = inter->cursor->planet->speed;
					printf("%s's engines upgraded to level %d.\n", 
							inter->cursor->ship->name,
							inter->cursor->ship->speed
							);
				}
			}
			inter->bars[BPRODUCTION] = inter->cursor->planet->production/10.0;
			break;
			
			
		case OPROPULSION:
			if (!inter->cursor->planet) {
				printf("Unable to develop new propulsion technology at (%d,%d):\n",
						inter->cursor->x, inter->cursor->y
						);
				printf("A planet is required do research.\n");
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("Unable to develop new propulsion technology on %s:\n",
						inter->cursor->planet->name
				      );
			    printf("You cannot direct the research efforts of planets you don't control.\n");
				return;
			}
			if (inter->cursor->planet->speed >= 16) {
				printf("Unable to develop new propulsion technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("The planet's industry and science cannot progress any further\n"
				       "in the field of propulsion systems.\n");
				return;
			}
			if (inter->cursor->planet->science < 100 * 1/inter->player->speed) {
				printf("Unable to develop new propulsion technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("%d science points are required.\n",(int)(100*1/inter->player->speed));
				return;
			}
			inter->cursor->planet->science -= 100 * 1/inter->player->speed;
			inter->cursor->planet->speed += 1;
			printf("The propulsion technology level attainable by the shipyards\n"
			       " of %s is now %d!\n", inter->cursor->planet->name, 
			       inter->cursor->planet->speed
			      );
			inter->bars[BPROPULSION] = inter->cursor->planet->speed * 10;
			inter->bars[BSCIENCE] = inter->cursor->planet->science / 10;
			break;
		case OWEAPONS:
			if (!inter->cursor->planet) {
				printf("Unable to develop new weapons technology at (%d,%d):\n",
						inter->cursor->x, inter->cursor->y
						);
				printf("A planet is required do research.\n");
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("Unable to develop new weapons technology on %s:\n",
						inter->cursor->planet->name
				      );
			    printf("You cannot direct the research efforts of planets you don't control.\n");
				return;
			}
			if (inter->cursor->planet->attack >= 16) {
				printf("Unable to develop new weapons technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("The planet's industry and science cannot progress any further\n"
				       "in the field of weaponry.\n");
				return;
			}
			if (inter->cursor->planet->science < 100 * 1/inter->player->attack) {
				printf("Unable to develop new weapon technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("%d science points are required.\n",(int)(100*1/inter->player->attack));
				return;
			}
			inter->cursor->planet->science -= 100 * 1/inter->player->attack;
			inter->cursor->planet->attack += 1;
			printf("The weapon technology level attainable by the shipyards\n"
			       " of %s is now %d!\n", inter->cursor->planet->name, 
			       inter->cursor->planet->attack
			      );
			inter->bars[BWEAPONS] = inter->cursor->planet->attack * 10;
			inter->bars[BSCIENCE] = inter->cursor->planet->science / 10;
			
			break;
		case ODEFENSES:
			if (!inter->cursor->planet) {
				printf("Unable to develop new defensive technology at (%d,%d):\n",
						inter->cursor->x, inter->cursor->y
						);
				printf("A planet is required do research.\n");
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("Unable to develop new defensive technology on %s:\n",
						inter->cursor->planet->name
				      );
			    printf("You cannot direct the research efforts of planets you don't control.\n");
				return;
			}
			if (inter->cursor->planet->defend >= 16) {
				printf("Unable to develop new defensive technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("The planet's industry and science cannot progress any further\n"
				       "in the field of defenses.\n");
				return;
			}
			if (inter->cursor->planet->science < 100 * 1/inter->player->defend) {
				printf("Unable to develop new defensive technology on %s:\n",
						inter->cursor->planet->name
				      );
				printf("%d science points are required.\n",(int)(100*1/inter->player->defend));
				return;
			}
			inter->cursor->planet->science -= 100 * 1/inter->player->defend;
			inter->cursor->planet->defend += 1;
			printf("The defense technology level attainable by the shipyards\n"
			       " of %s is now %d!\n", inter->cursor->planet->name, 
			       inter->cursor->planet->defend
			      );
			inter->bars[BDEFENSES] = inter->cursor->planet->defend * 10;
			inter->bars[BSCIENCE] = inter->cursor->planet->science / 10;
			
			break;

		case OTRANSPORT:
			if (!inter->cursor->planet) {
				printf("Unable to build a transport at (%d,%d):\n",
						inter->cursor->x, inter->cursor->y
				      );
				printf("A ship can only be built at a planet.\n");
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("Unable to build a transport on %s:\n",
						inter->cursor->planet->name
				      );
				printf("You can only build ships at planets you control.\n");
				return;
			}
			if (inter->cursor->ship) {
				printf("Unable to build a transport on %s:\n",
						inter->cursor->planet->name
				      );
				printf("The%s ship %s %s %s.\n",
					(inter->cursor->ship->race == inter->player?
					 	"" : inter->cursor->ship->race->name),
					inter->cursor->ship->name,
					(inter->cursor->ship->race == inter->player?
					 	"occupies the starport of" : 
						"is blockading"),
					inter->cursor->planet->name
				      );
				return;
			}

			if (inter->cursor->planet->production < TRANSPORT_COST) {
				printf("Unable to build a transport on %s:\n",
						inter->cursor->planet->name
				      );
				printf("Not enough production points are available. (125 are needed.)\n");
				return;
			}
			inter->cursor->planet->production -= TRANSPORT_COST;
			inter->bars[BPRODUCTION] = inter->cursor->planet->production/10.0;
			inter->cursor->ship = malloc(sizeof(Ship));
			inter->cursor->ship->attack = inter->cursor->planet->attack / 1.5;
			inter->cursor->ship->defend = inter->cursor->planet->defend / 1.5;
			inter->cursor->ship->speed = inter->cursor->ship->AP=inter->cursor->planet->speed/1.5;
			inter->cursor->ship->mHP = inter->cursor->ship->HP = inter->cursor->planet->defend*12;
			inter->cursor->ship->race = inter->player;
			inter->cursor->ship->id = inter->player->cship;
			inter->cursor->ship->name = inter->player->shipNames[(inter->player->cship++ + 8)%16];
			inter->cursor->ship->transport = 1;
			printf("The %s transport %s is launched from %s!\n", inter->player->name,
					inter->cursor->ship->name,
					inter->cursor->planet->name
			      );
		break;
		case OWARSHIP:
			if (!inter->cursor->planet) {
				printf("Unable to build a warship at (%d,%d):\n",
						inter->cursor->x, inter->cursor->y
				      );
				printf("A ship can only be built at a planet.\n");
				return;
			}
			if (inter->cursor->planet->race != inter->player) {
				printf("Unable to build a warship on %s:\n",
						inter->cursor->planet->name
				      );
				printf("You can only build ships at planets you control.\n");
				return;
			}
			if (inter->cursor->ship) {
				printf("Unable to build a warship on %s:\n",
						inter->cursor->planet->name
				      );
				printf("The %s ship %s %s %s.\n",
					(inter->cursor->ship->race == inter->player?
					 	"" : inter->cursor->ship->race->name),
					inter->cursor->ship->name,
					(inter->cursor->ship->race == inter->player?
					 	"occupies the starport of" : 
						"is blockading"),
					inter->cursor->planet->name
				      );
				return;
			}

			if (inter->cursor->planet->production < WARSHIP_COST) {
				printf("Unable to build a warship on %s:\n",
						inter->cursor->planet->name
				      );
				printf("Not enough production points are available. (100 are needed.)\n");
				return;
			}
			inter->cursor->planet->production -= WARSHIP_COST;
			inter->bars[BPRODUCTION] = inter->cursor->planet->production/10.0;
			inter->cursor->ship = malloc(sizeof(Ship));
			inter->cursor->ship->attack = inter->cursor->planet->attack;
			inter->cursor->ship->defend = inter->cursor->planet->defend;
			inter->cursor->ship->speed = inter->cursor->ship->AP = inter->cursor->planet->speed;
			inter->cursor->ship->mHP = inter->cursor->ship->HP = inter->cursor->planet->defend*10;
			inter->cursor->ship->race = inter->player;
			inter->cursor->ship->id = inter->player->cship;
			inter->cursor->ship->name = inter->player->shipNames[inter->player->cship++%16];
			inter->cursor->ship->transport = 0;
			printf("The %s warship %s is launched from %s!\n", inter->player->name,
					inter->cursor->ship->name,
					inter->cursor->planet->name
			      );
		break;
	


	}
}

void loadRaces(Race players[], FILE *file) {
	char name[80] = ""; 
	char planet[80]; char adjective[80];
	char image[80]; char shipfile[80];
	float attack, defend, speed, production, population;
	int id = 6, cloaks, strong;
	FILE *names;
	for (int i = 0; i < 4; ++i) {
		fscanf(file, "race %d, %[^,], %[^,], %[^,],  %[^,], %f, %f, %f, %f, %f, %d, %d, %[^ \n,] ",
	  	    &id, name, planet, adjective, image, &attack, &defend, &speed, &production, &population,
		    &cloaks, &strong, shipfile
		);
		players[i].name = malloc(strlen(name)+1);
		players[i].cship = 1;
		strcpy(players[i].name, name);
		players[i].adjective = malloc(strlen(adjective)+1);
		strcpy(players[i].adjective, adjective);
		players[i].planetName = malloc(strlen(planet));
		strcpy(players[i].planetName, planet);
		players[i].cloaks = cloaks;
		players[i].id = id;
		players[i].population = population;
		players[i].speed = speed;
		players[i].attack = attack;
		players[i].image = IMG_Load(image);
		players[i].planets = 0;
		players[i].defend = defend;
		players[i].strong = strong;
		players[i].production = production;
		names = fopen(shipfile, "r");
		if (!names) {
			fprintf(stderr, "Couldn't open '%s' to read %s ship names.\n",
					shipfile, adjective
			       );
			exit(1);
		}
		int tmp;
		for (int j = 0; j < 16; ++j) {
			fgets(name, 70, names);
			players[i].shipNames[j] = malloc(tmp=strlen(name));
			name[tmp-1] = '\0';
			strcpy(players[i].shipNames[j], name);
		}

	}
	//fscanf(file, "%30s", buffer);
	//printf("%s", buffer);
}



void renderInterface(SDL_Surface *target, Interface *inter) {
	SDL_Rect left = {0,0,192,640};
	SDL_Rect dest = {32*MSIZE, 0, 192, 640};
	SDL_BlitSurface(inter->graphic, &left, target, &dest);

	SDL_Rect bar = {208, 0, 160,24};
	SDL_Rect bdest = {656, 0, 160,24};
	for (int i = 0; i < 5; ++i) {
		bdest.w = inter->bars[i];
		bar.w = inter->bars[i];
		bar.y = i * 48 + 128;
		bdest.y = i * 48 + 128;
		SDL_BlitSurface(inter->graphic, &bar, target, &bdest);
	}
	if (inter->selected) {
		bar.w = bdest.w = 160; bar.h = bdest.h = 32;
		bar.x = 208; bar.y = bdest.y = 352 + inter->selected*32;
		bdest.x =  MSIZE*32 + 16;
		SDL_BlitSurface(inter->graphic, &bar, target, &bdest);
	}
	bdest.w = 128; bdest.h = 128; 
	bdest.x = MSIZE*32 + 16;
	bdest.y = 16;
	SDL_BlitSurface(inter->player->image, NULL, target, &bdest);
		
		
}

void scrollField(Interface *interface, SDLKey key) {
	switch (key) {
		case SDLK_UP:
			if (interface->y > 0)
				--interface->y;
			break;
		case SDLK_DOWN:
			if (interface->y < (interface->height - MSIZE))
				++interface->y;
			break;
		case SDLK_RIGHT:
			if (interface->x < (interface->width - MSIZE))
				++interface->x;
			break;
		case SDLK_LEFT:
			if (interface->x > 0)
				--interface->x;
			break;
	}
}

void centerMap(Interface *interface, Cell *cell) {
	interface->x = cell->x - MSIZE / 2;
	interface->y = cell->y - MSIZE / 2;
	if (interface->x < 0) 
		interface->x = 0;
	else if (interface->x > (interface->width - MSIZE))
		interface->x = interface->width - MSIZE;
	
	if (interface->y < 0) 
		interface->y = 0;
	else if (interface->y > (interface->height - MSIZE)) 
		interface->y = interface->height - MSIZE;
}
int main (int argc, char *argv[]) {
	SDL_Init(SDL_INIT_VIDEO);
	SDL_Surface *display = SDL_SetVideoMode(832, 640, 32, SDL_SWSURFACE);
	
	SDL_Surface *graphics = IMG_Load("graphics.png");
	SDL_Surface *iface = IMG_Load("interface.png");
	
	srand( time (NULL) );
	
	Race players[4];
	for (int i = 0; i < 4; ++i)
		players[i].id = i;
	loadRaces(players, fopen("data.dat", "r"));
	
	int currentPlayer = 0;
	
	Interface interface = {0,0,iface, 1, 0, players, {0}};
	
	Cell map[GSIZE][GSIZE];
	if (argc < 2)
		mkMap(map, fopen("default.map", "r"), players, &interface);
	else
		mkMap(map, fopen(argv[1], "r"), players, &interface);

	SDL_Event ev;
	render(display, graphics, map, &interface);
	renderInterface(display, &interface);
	SDL_Flip(display);

	while (1) {
		if (SDL_PollEvent(&ev)) {
			switch (ev.type) {
				case SDL_QUIT:
					exit(0);
				case SDL_KEYDOWN:
					
					switch (ev.key.keysym.sym) {
						case SDLK_UP:
						case SDLK_DOWN:
						case SDLK_LEFT:
						case SDLK_RIGHT:
							if (SDL_GetModState()&KMOD_CTRL) {
								scrollField(&interface, ev.key.keysym.sym);
							} else {
								if (!interface.cursor || !interface.cursor->ship 
									|| interface.cursor->ship->race != interface.player
									) {
									printf("You can only give movement orders to your ships.\n");
									break;
								}
								moveShip(map, interface.cursor, 
									ev.key.keysym.sym, &interface);
								break;
							}
					}
					render(display, graphics, map, &interface);
					renderInterface(display, &interface);
					SDL_UpdateRect(display,0,0,0,0);

					break;
				case SDL_MOUSEMOTION:
					if (ev.motion.x > 640 && ev.motion.y > 384 && ev.motion.y < 608) {
						interface.selected = (ev.motion.y-384)/32 + 1;
						renderInterface(display, &interface);
						SDL_UpdateRect(display, 640,0,192,640);
					}
					break;
				case SDL_MOUSEBUTTONDOWN:
					//SDL_PumpEvents();
					if (ev.button.button == SDL_BUTTON_LEFT) {
						if (ev.button.x <= 640)
							handleMapClick(map, &interface,
									ev.button.x/32, ev.button.y/32
									);
						else if (ev.button.y > 128)
							handleInterfaceClick(&interface, ev.button.y);
						else { // new turn
							do {
								++currentPlayer;
								currentPlayer %= 4;
							} while (!players[currentPlayer].planets);
							
							updateMap(map, &interface);
							printf("*It is the %s's turn. %s\n", players[currentPlayer].name,
									players[currentPlayer].homeworld->planet->name);
							interface.player = &players[currentPlayer];
							interface.cursor = players[currentPlayer].homeworld;
							if (interface.cursor) interface.cursor->selected = 0;
							centerMap(&interface, interface.cursor);
							interface.cursor->selected = 1;
							intr2cell(interface.cursor, &interface);
						}
					} else if (ev.button.button == SDL_BUTTON_WHEELDOWN) {
						if (SDL_GetModState()&KMOD_SHIFT) {
							scrollField(&interface, SDLK_LEFT);
						//	if (interface.x > 0)
						//		--interface.x;

						} else
							scrollField(&interface, SDLK_DOWN);
							//if (interface.y < (interface.height - MSIZE);
							//	++interface.y;
					} else if (ev.button.button == SDL_BUTTON_WHEELUP) {
						if (SDL_GetModState()&KMOD_SHIFT) {
							scrollField(&interface, SDLK_RIGHT);
						//	if (interface.x < (interface.width - MSIZE))
						//		++interface.x;
						} else
							scrollField(&interface, SDLK_UP);
						//	if (interface.y > 0)
						//		--interface.y;
					} else if (ev.button.button == SDL_BUTTON_MIDDLE) {
						interface.x = interface.cursor->x - MSIZE / 2;
						interface.y = interface.cursor->y - MSIZE / 2;
						if (interface.x < 0) interface.x = 0;
						else if (interface.x >= (interface.width - MSIZE)) interface.x = interface.width - MSIZE;
						if (interface.y < 0) interface.y = 0;
						else if (interface.y >= (interface.height - MSIZE)) interface.x = interface.height - MSIZE;

					} else if (ev.button.button = SDL_BUTTON_RIGHT) {
						gotoNextShip(map, interface.cursor, &interface);
						intr2cell(interface.cursor, &interface);
					}
					// FALL THROUGH
				case SDL_VIDEOEXPOSE:
					render(display, graphics, map, &interface);
					renderInterface(display, &interface);
					SDL_UpdateRect(display,0,0,0,0);
					break;
				

			}
		}
	}
	getchar();
}	
