#include "functions.h"

void printPlanet(Planet *planet) {
	if (!planet->race) {
		printf("%s, an unknown planet.\n", planet->name);
		return;
	}
	printf("The %s %s, %s.\nPopulation %f, Production %f, Research %f.\n",
		planet->race->adjective, 
		(planet->race->id == planet->id? "homeworld" : "colony" ),
		planet->name, 
		planet->population, planet->production, planet->science
	);
	printf("Weapons %d, Defense %d, Propulsion %d.\n",
			planet->attack, planet->defend, planet->speed
	      );
}
Planet *mkPlanet(Race *player) {
	Planet *p = malloc(sizeof(Planet));
	p->population = 40 * player->population;
	p->science = 0;
	if (player->production > 1.00)
		p->production += 50.0*player->production;
	else
		p->production = 0;
	p->attack = (player->attack*5);
	p->defend = (player->defend*5);
	++player->planets;
	p->speed = (player->speed*5);
	p->race = player;
	p->orders = researchDefend;
	p->id = player->id;
	p->name = player->planetName;
	player->homeworld = NULL;
	p->originalRace = player;
	return p;
}

void mkMap(Cell map[GSIZE][GSIZE], FILE *file, Race players[], Interface *intr) {
	char c1; int c2;
	int width, height;
	fscanf(file, "%d %d\n", &width, &height);
	intr->width = width; intr->height = height;
	if (width < 20 || width > 256 || height < 20 || height > 256) {
		fprintf(stderr, "Map size (%d,%d) is invalid.\n", width, height);
		exit(2);
	}
		
	
	for (int j = 0; j < height; ++j)
		for (int i = 0; i < width; ++i) {
			map[i][j].x = i;
			map[i][j].y = j;
			map[i][j].selected = 0;
			map[i][j].ship = NULL;
			map[i][j].planet = NULL;
			map[i][j].special.type = none;
			c1 = fgetc(file); 
			if (c1 == '\n') c1 = fgetc(file);
			c2 = fgetc(file) - '0';
			switch (c1) {
				case '0': case '1': case '2':
				case '3': case '4': case '5':
				case '6': case '7': case '8': case '9':
					map[i][j].ship = mkShip(&players[
							c1-'0'], c2
							);
					
				break;
				case 'P':
					map[i][j].planet = mkPlanet(
							&players[c2]
							);
					players[c2].homeworld = &map[i][j];
				break;
				case 'C':
					map[i][j].planet = malloc(sizeof(Planet));
					map[i][j].planet->population = 10*c2;
					map[i][j].planet->science = 0;
					map[i][j].planet->production = 0;
					map[i][j].planet->originalRace = NULL;
					map[i][j].planet->id = -(1+rand()%4);
					map[i][j].planet->name = Gnames[rand()%GLEN];
					map[i][j].planet->race = NULL;
					map[i][j].planet->attack = map[i][j].planet->defend = map[i][j].planet->speed = c2;
				break;
				case '*':
					map[i][j].special.type = nebula;
					map[i][j].special.param = c2;
				break;
				case 'W':
					map[i][j].special.type = wormhole;
					map[i][j].special.param = c2;
				break;
				case '#':
					while (fgetc(file) != '\n')
						;
				break;
			}
		}
	
}


void handleMapClick(Cell map [GSIZE][GSIZE], Interface *inter, int x, int y) {
	Cell *cl = &map[x+inter->x][y+inter->y];
	if (inter->cursor) inter->cursor->selected = 0;
	inter->cursor = cl;
	cl->selected = 1;
	for (int i = 0; i < 5; ++i)
		inter->bars[i] = 0;
	printf("Clicked on map cell (%d,%d). Contains:\n", x, y);
	if (cl->ship) {
		printShip(cl->ship);
		inter->bars[BPROPULSION] = cl->ship->speed*10;
		inter->bars[BWEAPONS] = cl->ship->attack*10;
		inter->bars[BDEFENSES] = cl->ship->defend*10;
	} 
	if (cl->planet) {
		printPlanet(cl->planet);
		inter->bars[BPROPULSION] = cl->planet->speed*10;
		inter->bars[BWEAPONS] = cl->planet->attack*10;
		inter->bars[BDEFENSES] = cl->planet->defend*10;
		inter->bars[BPRODUCTION] = cl->planet->production/10.0;
		inter->bars[BSCIENCE] = cl->planet->science/10.0;
	}

	if (cl->special.type == nebula)
		printf("A level %d nebular cloud.\n", cl->special.param);
	else if (cl->special.type == wormhole)
		printf("Type %d wormhole.\n", cl->special.param);
	if (!cl->ship && !cl->planet && cl->special.type == none)
		printf("Empty space.\n");

}

// Update planet
// 
void planet (Cell *cell) {
	if (!cell->planet->race && (!cell->ship || !cell->ship->transport)) return;
	
	if (cell->ship && cell->ship->transport
		&& cell->planet->race && cell->ship->race != cell->planet->race){
		printf("\x1b[31;43;m* The %s invades %s!\x1b[0;m\n", cell->ship->race->name,
				cell->planet->name
		      );
		cell->ship->HP -= cell->planet->attack*2;
		if (cell->ship->HP <= 0) {
			printf("  The invasion has been repelled.\n");
		} else {
			if (cell->planet->defend && rand()%2) cell->planet->defend -= 1;
			if (cell->planet->attack && rand()%2) cell->planet->attack -= 1;
			if (cell->planet->speed && rand()%2) cell->planet->speed -= 1;
			
			cell->planet->population -= cell->ship->attack*2;
		       printf("  %s ground forces damage infastructure and reduce %s's population to %f.\n",
				cell->ship->race->adjective,
				cell->planet->name,
				cell->planet->population
			      );
		       if (cell->planet->race->strong && rand()%2) {
			       printf("  %s commanders report unusually fierce resistance by %s defenders of %s!\n",
					       cell->ship->race->adjective, cell->planet->race->adjective,
					       cell->planet->name
				     );
			       cell->ship->HP -= 10;
		       }
		       if (cell->planet->race->strong > 1 && cell->planet->population < 40 && !(rand()%3)) {
			       printf("%s insurgents detonate a bomb on %s! The ship is lost!\n",
					       cell->planet->race->name,
					       cell->ship->name
					       );
			       cell->ship->HP = -10;
		       }


		}
		if (cell->planet->population <= 0) {
			printf("\x1b[33;mThe %s military captures %s!\x1b[0;m\n", cell->ship->race->adjective,
					cell->planet->name
			      );
			--cell->planet->race->planets;
			++cell->ship->race->planets;
			if (!cell->planet->race->planets) 
			    printf("The %s has exterminated the %s species.\n",
					  cell->ship->race->name,
					  cell->planet->race->adjective
				  );
			cell->planet->race = cell->ship->race;
			cell->planet->population=2*cell->planet->race->population;
		}
	}
	if (cell->ship && cell->ship->transport && !cell->planet->race) {
		printf("The %s colonize %s.\n", 
				cell->ship->race->name,
				cell->planet->name
			      );
		cell->planet->originalRace = cell->ship->race;
		cell->planet->attack = cell->ship->attack / 2;
		cell->planet->speed = cell->ship->speed / 2;
		cell->planet->defend = cell->ship->defend / 2;
		++cell->ship->race->planets;
		cell->planet->race = cell->ship->race;
		cell->planet->population=4*cell->planet->race->population;
	}
	if (cell->ship && cell->planet->race == cell->ship->race 
			&& cell->ship->HP != cell->ship->mHP && cell->ship->HP > 0) {
		printf("The %s is repaired on %s.\n", 
				cell->ship->name,
				cell->planet->name
			      );
		cell->ship->HP = cell->ship->mHP;
	}
	cell->planet->production += 
		(cell->planet->population*cell->planet->race->production)/4;
	cell->planet->science += cell->planet->race->planets * 10;
	cell->planet->population += cell->planet->race->population;

	// Insurgent coded
	if (cell->planet->race != cell->planet->originalRace) {
		if ((cell->planet->originalRace->strong > cell->planet->race->strong
		  && !(rand()%8)) || !(rand()%32)) {
			printf("\x1b[33;mAsymetric Warfare on %s!\x1b[0;m ", cell->planet->name);
			printf(Grebels[rand()%TLEN], cell->planet->originalRace->adjective);
			switch (rand()%6) {
				case 0:
					printf(" released a targeted bioweapon!\n");
					cell->planet->population /= (rand()%4 + 2);
					break;
				case 1:
					printf(" sabotaged industrial production!\n");
					cell->planet->production = 0;
					break;
				case 2:
					printf(" abducted %s scientists!\n", cell->planet->race->adjective);
					cell->planet->science = 0;
					break;
				case 3:
					printf(" have incited an insurrection against %s authorities!\n",
							cell->planet->race->name
					      );
					cell->planet->race = cell->planet->originalRace;
					
					break;
				case 4:
					if (cell->ship) {
						printf(" detonated a bomb on the %s!\n", cell->ship->name);
						cell->ship->HP = -10;
					} else {
						printf(" encouraged riots and civil unrest.\n");
						cell->planet->population /= 1.1;
					}
					break;
				case 5:
					printf(" seriously sabotaged the planet's shipyards!\n");
					cell->planet->attack /= 1.1 + 0.01*(rand()%100);
					cell->planet->defend /= 1.1 + 0.01*(rand()%100);
					cell->planet->speed /= 1.1 + 0.01*(rand()%100);
					break;
			}
		}
	}

}


void render(SDL_Surface *target, SDL_Surface *graphics, Cell map[][GSIZE], Interface *inter) {
	SDL_Rect rect = {0,0,32,32}, trect = {0,0,32,32};
	int j, i;
	for (i = inter->x, trect.x = 0; i < inter->x+MSIZE; ++i, trect.x += 32)
		for (j = inter->y, trect.y = 0; j < inter->y+MSIZE; ++j, trect.y += 32) {
			SELECT(0,1);
			SDL_BlitSurface(graphics, &rect, target, &trect);
			if (map[i][j].planet) {
				if (map[i][j].planet->id >= 0)
					SELECT(map[i][j].planet->id, 0);
				else
					SELECT((-map[i][j].planet->id)  - 1, 5);
				SDL_BlitSurface(graphics, &rect, target,&trect);
			}
			if (map[i][j].special.type == wormhole) {
				SELECT(3,1);
				SDL_BlitSurface(graphics, &rect, target,&trect);
			} else if (map[i][j].special.type == nebula) {
				SELECT(2,1);
				SDL_BlitSurface(graphics, &rect, target,&trect);
			}
			
				
			if (map[i][j].ship) {
				if (map[i][j].ship->transport)
					SELECT(map[i][j].ship->race->id,3);
				else
					SELECT(map[i][j].ship->race->id,2);
				SDL_BlitSurface(graphics, &rect, target,&trect);
			}
			if (map[i][j].selected) {
				if (!map[i][j].ship && !map[i][j].planet) 
					SELECT(1,1);
				if (map[i][j].planet)
					if (map[i][j].planet->race == inter->player)
						SELECT(0,4);
					else if (!map[i][j].planet->race)
						SELECT(1,4);
					else
						SELECT(2,4);
						
				if (map[i][j].ship){
					if (map[i][j].ship->race == inter->player)
						SELECT(0,4);
					else
						SELECT(2,4);
					SDL_BlitSurface(graphics, &rect, target,&trect);
						
					SELECT(3,4);
					trect.w = rect.w = (
					  (float)(map[i][j].ship->HP)/map[i][j].ship->mHP
						) * 32;
					if (trect.w > 32) trect.w = rect.w = 0;
					SDL_BlitSurface(graphics, &rect, target,&trect);
					SELECT(1,4);
					trect.w = rect.w = (
					  (float)(map[i][j].ship->AP)/map[i][j].ship->speed
						) * 32;
					if (trect.w > 32) trect.w = rect.w = 0;

				}
				SDL_BlitSurface(graphics, &rect, target,&trect);
				trect.w = rect.w = 32;
			}


		}

}
