#include "functions.h"
void moveShip(Cell map [GSIZE][GSIZE], Cell *cell, SDLKey key, Interface *intr) {
	Cell *tcell;
	switch (key) {
		case SDLK_UP:
			if (cell->y <= 0) return;
			tcell =  &map[cell->x][cell->y-1];
			break;
		case SDLK_DOWN:
			if (cell->y >= GSIZE - 1) return;
			tcell =  &map[cell->x][cell->y+1];
			break;
		case SDLK_LEFT:
			if (cell->x <= 0) return;
			tcell =  &map[cell->x-1][cell->y];
			break;
		case SDLK_RIGHT:
			if (cell->x >= GSIZE - 1) return;
			tcell =  &map[cell->x+1][cell->y];
			break;
	}
	if (cell->ship->AP <= 0) {
		printf("The %s can't move anymore on this turn.\n",
				cell->ship->name
			);
		return;
	}
	
	if (!tcell->ship) { // move
		if (tcell->special.type == nebula) {
			if (tcell->special.param > cell->ship->AP) {
			  printf("The %s is unable to navigate that anomally.\n",
					  cell->ship->name
				);
			  printf("It doesn't have enough moves available.\n");
			  return;
			} else {
				tcell->ship = cell->ship;
				cell->ship->AP -= tcell->special.param;
				cell->ship = NULL;
			}
			//handleMapClick(map, i, tcell->x, tcell->y);

		} else if (tcell->special.type == wormhole) {
			printf("%s travels through a wormhole!\n", 
					cell->ship->name
			      );
			--cell->ship->AP;
			for (int i = 0; i < GSIZE; ++i)
				for (int j = 0; j < GSIZE; ++j)
					if (map[i][j].special.type == wormhole && 
							map[i][j].special.param == tcell->special.param 
							&& &map[i][j] != tcell) 
						if (!map[i][j].ship) {
							tcell = &map[i][j];
							map[i][j].ship = cell->ship;
							cell->ship = NULL;
							centerMap(intr, tcell);
							goto LOOP_DONE;
						} else {
							printf("%s lost in the wormhole!\n", cell->ship->name);
							free(cell->ship);
							cell->ship  = NULL;
							centerMap(intr, tcell);
							goto LOOP_DONE;
						}
			printf("Ship lost in this unstable wormhole!\n");
			free(cell->ship);
			cell->ship = NULL;
		} else {
			--cell->ship->AP;
			tcell->ship = cell->ship;
			cell->ship = NULL;
			//handleMapClick(map, i, tcell->x, tcell->y);
		}
	} else if (tcell->ship->race != cell->ship->race && cell->ship->AP) {
		// combat
		printf("%s attacks %s!\n", cell->ship->name, tcell->ship->name);
		combat(cell, tcell);
		tcell = cell;
	}
LOOP_DONE:
	if (intr->cursor) intr->cursor->selected = 0;
	intr->cursor = tcell;
	tcell->selected = 1;

}
void printShip(Ship *ship) {
	printf("%s %s %s, ID %d.\n",
		ship->race->adjective,  
		(ship->transport? "transport" : "warship"),
		ship->name, ship->id
		);
	printf("HP %d/%d, AP: %d, ATK: %d, DEF %d, SPD %d%s\n",
			ship->HP, ship->mHP, ship->AP, ship->attack,
			ship->defend, ship->speed,
			(ship->race->cloaks? " Steath Shielded":"")
	      );

}

Ship *mkShip(Race *player, int id) {
	Ship *ship = malloc(sizeof(Ship));
	ship->attack = player->attack*6;
	ship->defend = player->defend*6;
	ship->speed = ship->AP = player->speed*6;
	ship->race = player;
	ship->HP = ship->mHP = player->defend*100;
	ship->id = id;
	ship->name = player->shipNames[id];
	ship->transport = 0;
	return ship;
}

void ship(Cell *cell) {
	cell->ship->AP = cell->ship->speed;
	if (cell->ship->HP <= 0) {
		printf("%s %s %s destroyed.\n",
				cell->ship->race->adjective,
				cell->ship->transport? "transport" : "warship",
				cell->ship->name
		      );
		free(cell->ship);
		cell->ship = NULL;
	}
}

void combat (Cell *catk, Cell *cdef) {
	Ship *atk = catk->ship;
	Ship *def = cdef->ship;
	if (def->HP < 0) {
		printf("The %s is already dead. While it may be cathartic,"
			" attacking it\nagain is not rational.\n", def->name
			);
		return;
	}
	--def->AP; --atk->AP;
	cdef->selected = 1;
	if (def->race->cloaks && !(rand()%2) && !cdef->planet) {
		printf("The enemy has evaded our forces by stealth!\n");
		return;
	}
	
	int damage = (def->attack*(rand()%80+20))/10.0 - atk->defend;
	if (damage <= 0) damage = 1;
	printf("The %s inflicts %d damage on the %s.\n",
			atk->name, damage, def->name
	      );
	def->HP -= damage;
	if (def->AP && def->speed >= atk->speed) {
		damage = (def->attack*(rand()%80+20))/10.0 - atk->defend;
		if (damage <= 0) damage = 1;
		printf("The %s counter-attacks, doing %d damage.\n",
				def->name, damage
				);
		atk->HP -= damage;
	}

	if (atk->HP < 0) {
		printf("Our ship was destroyed by the counter-attack!\n");
		free(atk);
		catk->ship = NULL;
	}
	if (def->HP < 0) {
		printf("The enemy ship was destroyed by our attack!\n");
		free(def);
		cdef->selected = 0;
		cdef->ship = NULL;
	}
	


}

void gotoNextShip(Cell map[GSIZE][GSIZE], Cell *current, Interface *intr) {
	if (!current) return;
	int w = intr->width, h = intr->height;
	for (int i = current->x; i < w+current->x; ++i)
		for (int j = current->y; j < h+current->y; ++j)
			if (    map[i%w][j%h].ship && map[i%w][j%h].ship != current->ship
				&& map[i%w][j%h].ship->race == intr->player 
				&& map[i%w][j%h].ship->AP == map[i%w][j%h].ship->speed
				) {
				if (intr->cursor) intr->cursor->selected = 0;
				intr->cursor = &map[i%w][j%h];
				intr->cursor->selected = 1;
				if (i%w >= intr->x+MSIZE || i%w < intr->x 
				    || j%h >= intr->y+MSIZE || j%h < intr->y)
					centerMap(intr, &map[i%w][j%h]);
				
				goto BREAK;
			}
BREAK:
	return;
}

