#ifndef STRUCTS_H
#define STRUCTS_H
#include <SDL/SDL.h>
#include <stdlib.h>
#define MSIZE 20
#define GSIZE 256
// Max galaxy size
#define BPROPULSION 0
#define BWEAPONS 1
#define BDEFENSES 2
#define BPRODUCTION 3
#define BSCIENCE 4

#define TRANSPORT_COST 125
#define WARSHIP_COST 100

#define OTRANSPORT 1
#define OWARSHIP 2
#define OPROPULSION 3
#define OWEAPONS 4
#define ODEFENSES 5
#define OUPGRADE 6
#define OSCRAP 7

typedef struct Interface {
	int x, y;
	SDL_Surface *graphic;
	int selected;
	int planet; // Planet or a ship?
	struct Race *player; // Who's turn is it?
	int bars[5]; // Values of the bars
	struct Cell *cursor;
	int width, height;
} Interface;


typedef enum Order {
	buildTransport, buildWarship, researchAttack, researchDefend,
	researchSpeed, upgrade
} Order;

typedef enum SpecialType {
	none, nebula, wormhole
} SpecialType;

typedef struct Special {
	SpecialType type;
	long param;
} Special;

typedef struct Race {
	// Race modifiers
	float population, attack, defend, speed, production;
	char *name, *adjective, *planetName;
	char *shipNames[16];
	int cship;
	int cloaks;
	int strong; // Good ground fighter?
	int id;
	int planets;
	struct Cell *homeworld;
	SDL_Surface *image; // Planet graphic
} Race;

typedef struct Ship {
	int attack, defend, speed;
	int mHP, HP;
	int AP;
	Race *race;
	const char *name;
	int id;
	int transport;
} Ship;



typedef struct Planet {
	// Current population
	float population;
	// Accumulated science points
	float science;
	// Accumulated production
	float production;

	// tech levels
	int attack, defend, speed;
	int id;
	Order orders;
	char *name;
	Race *race;
	Race *originalRace;
} Planet;

typedef struct Cell {
	int x, y;
	Planet *planet;
	int selected;
	Ship *ship;
	Special special;
} Cell;



#ifdef MAIN
char *Gnames[] = {
	"Bathyos",
	"Ryzome",
	"Gulgaria",
	"Karn",
	"Trade World",
	"Binpinardia",
	"Canis Lupis",
	"Lumini 8",
	"Sandis O'Lait",
	"Planet X",
	"Planet Z",
	"Gargantia"
};
char *Grebels[] = {
	"The %s Liberation Front has",
	"The %s Brotherhood has",
	"The %s Freedom Organization has",
	"The Army of %s Defenders have",
	"The %s Liberation Army has",
	"%s terrorists have",
	"%s rebels have",
	"The %s insurgency has"
};
#else
extern char *Gnames[];
extern char *Grebels[];
#endif
#define GLEN 12
#define TLEN 8
#endif
